#ifndef ChiNu_h
#define ChiNu_h 1
/*****************************************************************************
 * Copyright (C) 2009-2019   this file is part of the NPTool Project       *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Pierre Morfouace  contact address: pierre.morfouace2@cea.fr                        *
 *                                                                           *
 * Creation Date  : February 2019                                           *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 *  This class describe  ChiNu simulation                             *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *****************************************************************************/

// C++ header
#include <string>
#include <vector>
using namespace std;

// G4 headers
#include "G4ThreeVector.hh"
#include "G4RotationMatrix.hh"
#include "G4LogicalVolume.hh"
#include "G4AssemblyVolume.hh"
#include "G4MultiFunctionalDetector.hh"

// NPTool header
#include "NPSVDetector.hh"
#include "TChiNuData.h"
#include "NPInputParser.h"

class ChiNu : public NPS::VDetector{
  ////////////////////////////////////////////////////
  /////// Default Constructor and Destructor /////////
  ////////////////////////////////////////////////////
  public:
    ChiNu() ;
    virtual ~ChiNu() ;

    ////////////////////////////////////////////////////
    /////// Specific Function of this Class ///////////
    ////////////////////////////////////////////////////
  public:
    // Cartesian
  void AddDetector(G4ThreeVector POS, double TimeResolution);
    // Spherical
  void AddDetector(double R,double Theta,double Phi, double TimeResolution);  


    G4AssemblyVolume* BuildDetector();
 
  private:
    G4LogicalVolume* m_CylindricalDetector;
    G4LogicalVolume* m_PMT;
    G4LogicalVolume* m_LightGuide;
    G4LogicalVolume* m_LeadShield;
    G4AssemblyVolume* m_AssemblyVolume;

    
    ////////////////////////////////////////////////////
    //////  Inherite from NPS::VDetector class /////////
    ////////////////////////////////////////////////////
  public:
    // Read stream at Configfile to pick-up parameters of detector (Position,...)
    // Called in DetecorConstruction::ReadDetextorConfiguration Method
    void ReadConfiguration(NPL::InputParser) ;

    // Construct detector and inialise sensitive part.
    // Called After DetecorConstruction::AddDetector Method
    void ConstructDetector(G4LogicalVolume* world) ;

    // Add Detector branch to the EventTree.
    // Called After DetecorConstruction::AddDetector Method
    void InitializeRootOutput() ;

    // Read sensitive part and fill the Root tree.
    // Called at in the EventAction::EndOfEventAvtion
    void ReadSensitive(const G4Event* event) ;

  public:   // Scorer
    //   Initialize all Scorer used by the MUST2Array
    void InitializeScorers() ;

    //   Associated Scorer
    G4MultiFunctionalDetector* m_ChiNuScorer ;
    ////////////////////////////////////////////////////
    ///////////Event class to store Data////////////////
    ////////////////////////////////////////////////////
  private:
    TChiNuData* m_Event ;

    ////////////////////////////////////////////////////
    ///////////////Private intern Data//////////////////
    ////////////////////////////////////////////////////
  private: // Geometry
    // Detector Coordinate 
    vector<double>  m_R; 
    vector<double>  m_Theta;
    vector<double>  m_Phi; 
    bool m_BuildLeadShield;
    vector<double>  m_ResoTime; 
   
    // Visualisation Attribute
    G4VisAttributes* m_VisCylinder;
    G4VisAttributes* m_VisPMT;
    G4VisAttributes* m_VisLightGuide;
    G4VisAttributes* m_VisPyrex;
    G4VisAttributes* m_VisLeadShield;
    G4VisAttributes* m_VisFCWall;
    G4VisAttributes* m_VisAl;
    G4VisAttributes* m_VisCu;
    G4VisAttributes* m_VisTi;
    G4VisAttributes* m_VisRogers4003C;

  // Needed for dynamic loading of the library
  public:
    static NPS::VDetector* Construct();
};
#endif
