/*****************************************************************************
 * Copyright (C) 2009-2016   this file is part of the NPTool Project         *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Adrien MATTA  contact address: matta@lpccaen.in2p3.fr    *
 *                                                                           *
 * Creation Date  : February 2013                                            *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 *  File old the scorer specific to the Sharc Detector                       *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 * This new type of scorer is aim to become the standard for DSSD,SSSD and   *
 * PAD detector (any Silicon Detector)                                       *
 *****************************************************************************/
#include "GaseousDetectorScorers.hh"
#include "G4UnitsTable.hh"
using namespace GaseousDetectorScorers;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
unsigned int
GaseousDetectorData::CalculateIndex(const vector<unsigned int>& level) {

  unsigned int size       = level.size();
  unsigned int result     = 0;
  unsigned int multiplier = 1;
  for (unsigned int i = 0; i < size; i++) {
    result += level[i] * multiplier;
    multiplier *= 1000;
  }
  return result;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
vector<GaseousDetectorData>::iterator
GaseousDetectorDataVector::find(const unsigned int& index) {
  for (vector<GaseousDetectorData>::iterator it = m_Data.begin();
       it != m_Data.end(); it++) {
    if ((*it).GetIndex() == index)
      return it;
  }
  return m_Data.end();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
PS_GaseousDetector::PS_GaseousDetector(G4String name, vector<G4int> NestingLevel,
                               G4int depth)
    : G4VPrimitiveScorer(name, depth) {
  m_NestingLevel = NestingLevel;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
PS_GaseousDetector::~PS_GaseousDetector() {}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
G4bool PS_GaseousDetector::ProcessHits(G4Step* aStep, G4TouchableHistory*) {

  // Contain Particle Id, Energy, Time and as many copy number as nested volume
  unsigned int mysize = m_NestingLevel.size();
  string particlename = aStep->GetTrack()->GetParticleDefinition()->GetParticleName();
  double   slice_posZ = aStep->GetPreStepPoint()->GetPosition().z(); 
  
  t_Energy = aStep->GetTotalEnergyDeposit();
  t_Time   = aStep->GetPreStepPoint()->GetGlobalTime();
  t_ParticleName.push_back(particlename);
  t_SlicePosZ.push_back(slice_posZ);
  t_EnergyLossPerSlice.push_back(t_Energy);
  //cout << "mysize = " << mysize << endl;
  //cout << "t_ParticleName : " << particlename << endl;
  //cout << "t_Energy       : " << t_Energy << endl;
  //cout << "slice_posZ     : " << slice_posZ << endl; // it seems that step_size is not uniform
  //cout << "t_ParticleName.size()       = " << t_ParticleName.size() << endl;
  //cout << "t_SlicePosZ.size()          = " << t_SlicePosZ.size() << endl;
  //cout << "t_EnergyLossPerSlice.size() = " << t_EnergyLossPerSlice.size() << endl;
  t_Level.clear();
  for (unsigned int i = 0; i < mysize; i++) {
    t_Level.push_back(
        aStep->GetPreStepPoint()->GetTouchableHandle()->GetCopyNumber(
            m_NestingLevel[i]));
  }
  // Check if the particle has interact before, if yes, add up the energies.
  vector<GaseousDetectorData>::iterator it;
  it = m_Data.find(GaseousDetectorData::CalculateIndex(t_Level));
  if (it != m_Data.end()) {
    it->Add(t_Energy);
    it->SetParticleName(particlename);
    it->SetSlicePosZ(slice_posZ);
    it->SetEnergyLossPerSlice(t_Energy);
  } else {
    //m_Data.Set(t_Energy, t_Time, t_Level, t_ParticleName, t_SliceNbr, t_SlicePosZ, t_EnergyLossPerSlice);
    m_Data.Set(t_Energy, t_Time, t_Level, t_ParticleName, t_SlicePosZ, t_EnergyLossPerSlice);
  }
  return TRUE;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void PS_GaseousDetector::Initialize(G4HCofThisEvent*) { clear(); }

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void PS_GaseousDetector::EndOfEvent(G4HCofThisEvent*) {}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void PS_GaseousDetector::clear() {
  m_Data.clear();
  t_Level.clear();
  t_ParticleName.clear();
  //t_SliceNbr.clear();
  t_SlicePosZ.clear();
  t_EnergyLossPerSlice.clear();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void PS_GaseousDetector::DrawAll() {}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void PS_GaseousDetector::PrintAll() {}
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
